@@ -14,15 +14,20 @@ import matplotlib.pyplot as plt
 
 from pgmpy.estimators import BDeuScore, K2Score, BicScore
 from pgmpy.estimators import ExhaustiveSearch, HillClimbSearch
-from pgmpy.estimators import ConstraintBasedEstimator
-# from pgmpy.estimators import PC as ConstraintBasedEstimator
+
+import pgmpy
+from packaging import version
+if version.parse(pgmpy.__version__)>=version.parse("0.1.13"):
+    from pgmpy.estimators import PC as ConstraintBasedEstimator
+else:
+    from pgmpy.estimators import ConstraintBasedEstimator
 
 # from packaging import version
 from bnlearn.bnlearn import _dag2adjmat
 
 
 # %% Structure Learning
-def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None, bw_list_method='enforce', max_indegree=None, epsilon=1e-4, max_iter=1e6, verbose=3):
+def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None, bw_list_method='enforce', max_indegree=None, epsilon=1e-4, max_iter=1e6, root_node=None, verbose=3):
     """Structure learning fit model.
 
     Description
@@ -34,6 +39,7 @@ def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None,
         1. Score-based structure learning (BIC/BDeu/K2 score; exhaustive search, hill climb/tabu search)
             a. exhaustivesearch
             b. hillclimbsearch
+            c. chow-liu
         2. Constraint-based structure learning (PC)
             a. chi-square test
         3. Hybrid structure learning (The combination of both techniques) (MMHC)
@@ -55,6 +61,7 @@ def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None,
         'hc' or 'hillclimbsearch' (default)
         'ex' or 'exhaustivesearch'
         'cs' or 'constraintsearch'
+        'cl' or 'chow-liu' (requires setting root_node parameter)
     scoretype : str, (default : 'bic')
         Scoring function for the search spaces.
         'bic', 'k2', 'bdeu'
@@ -73,6 +80,8 @@ def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None,
         Defines the exit condition. If the improvement in score is less than `epsilon`, the learned model is returned. (only in case of methodtype='hc')
     max_iter: int (default: 1e6)
         The maximum number of iterations allowed. Returns the learned model when the number of iterations is greater than `max_iter`. (only in case of methodtype='hc')
+    root_node: String
+        The root node for chow-liu method.
     verbose : int, (default : 3)
         Print progress to screen.
         0: NONE
@@ -111,12 +120,12 @@ def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None,
     """
     assert isinstance(pd.DataFrame(), type(df)), 'df must be of type pd.DataFrame()'
     assert (scoretype=='bic') | (scoretype=='k2') | (scoretype=='bdeu'), 'scoretype must be string: "bic", "k2" or "bdeu"'
-    assert (methodtype=='hc') | (methodtype=='ex')| (methodtype=='cs') | (methodtype=='exhaustivesearch')| (methodtype=='hillclimbsearch')| (methodtype=='constraintsearch'), 'Methodtype string is invalid'  # noqa
+    assert (methodtype=='cl') | (methodtype=='chow-liu') | (methodtype=='hc') | (methodtype=='ex') | (methodtype=='cs') | (methodtype=='exhaustivesearch')| (methodtype=='hillclimbsearch')| (methodtype=='constraintsearch'), 'Methodtype string is invalid'  # noqa
     if isinstance(white_list, str): white_list = [white_list]
     if isinstance(black_list, str): black_list = [black_list]
     if (white_list is not None) and len(white_list)==0: white_list = None
     if (black_list is not None) and len(black_list)==0: black_list = None
-    if (bw_list_method is None) : bw_list_method='enforce'
+    if (bw_list_method is None): bw_list_method='enforce'
 
     config = {}
     config['verbose'] = verbose
@@ -128,11 +137,13 @@ def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None,
     config['max_indegree'] = max_indegree
     config['epsilon'] = epsilon
     config['max_iter'] = max_iter
+    config['root_node'] = root_node
 
     # Show warnings
     # PGMPY_VER = version.parse(pgmpy.__version__)>version.parse("0.1.9")  # Can be be removed if pgmpy >v0.1.9
     # if (not PGMPY_VER) and ((black_list is not None) or (white_list is not None)):
-        # if config['verbose']>=2: print('[bnlearn] >Warning: black_list and white_list only works for pgmpy > v0.1.9')  # Can be be removed if pgmpy >v0.1.9
+    # if config['verbose']>=2: print('[bnlearn] >Warning: black_list and white_list only works for pgmpy > v0.1.9')  # Can be be removed if pgmpy >v0.1.9
+
     if df.shape[1]>10 and df.shape[1]<15:
         if config['verbose']>=2: print('[bnlearn] >Warning: Computing DAG with %d nodes can take a very long time!' %(df.shape[1]))
     if (black_list is not None) and methodtype!='hc':
@@ -182,6 +193,11 @@ def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None,
         Independencies in the data can be identified using chi2 conditional independence tests."""
         out = _constraintsearch(df, verbose=config['verbose'])
 
+    # Constraint-based Structure Learning
+    if config['method']=='cl' or config['method']=='chow-liu':
+        """Chow-liu based Structure Learning."""
+        out = _chowliu(df, config['root_node'], verbose=config['verbose'])
+
     # Setup simmilarity matrix
     adjmat = _dag2adjmat(out['model'])
 
@@ -224,6 +240,27 @@ def _white_black_list(df, white_list, black_list, bw_list_method='enforce', verb
     return df
 
 
+# %% hillclimbsearch
+def _chowliu(df, root_node, verbose=3):
+    """Chow-liu searches for DAGs with attempts to find a model with optimal score.
+
+    Description
+    -----------
+
+    """
+    from pgmpy.estimators import TreeSearch
+
+    out={}
+    est = TreeSearch(df, root_node=root_node)
+    model = est.estimate(estimator_type="chow-liu")
+
+    # Store
+    out['model']=model
+    out['model_edges']=model.edges()
+    # Return
+    return(out)
+
+
 # %% Constraint-based Structure Learning
 def _constraintsearch(df, significance_level=0.05, verbose=3):
     """Contrain search.
@@ -235,7 +272,7 @@ def _constraintsearch(df, significance_level=0.05, verbose=3):
     given the null hypothesis that X and Y are independent given Zs.
     This can be used to make independence judgements, at a given level of significance.
     """
-    out=dict()
+    out = {}
     # Set search algorithm
     model = ConstraintBasedEstimator(df)
 
@@ -254,12 +291,12 @@ def _constraintsearch(df, significance_level=0.05, verbose=3):
         Step 1.&2. form the so-called PC algorithm, see [2], page 550. PDAGs are `DirectedGraph`s, that may contain both-way edges, to indicate that the orientation for the edge is not determined.
     """
     # Estimate using chi2
-    [skel, seperating_sets] = model.estimate_skeleton(significance_level=significance_level)
+    [skel, seperating_sets] = model.build_skeleton(significance_level=significance_level)
 
     print("Undirected edges: ", skel.edges())
     pdag = model.skeleton_to_pdag(skel, seperating_sets)
     print("PDAG edges: ", pdag.edges())
-    dag = model.pdag_to_dag(pdag)
+    dag = pdag.to_dag()
     print("DAG edges: ", dag.edges())
 
     out['undirected'] = skel
@@ -311,9 +348,9 @@ def _hillclimbsearch(df, scoretype='bic', black_list=None, white_list=None, max_
     edges or to limit the search.
 
     """
-    out=dict()
+    out={}
     # Set scoring type
-    scoring_method = _SetScoringType(df, scoretype)
+    scoring_method = _SetScoringType(df, scoretype, verbose=verbose)
     # Set search algorithm
     model = HillClimbSearch(df, scoring_method=scoring_method)
 
@@ -329,10 +366,10 @@ def _hillclimbsearch(df, scoretype='bic', black_list=None, white_list=None, max_
         if (black_list is not None) or (white_list is not None):
             if verbose>=3: print('[bnlearn] >Enforcing nodes based on black_list and/or white_list.')
         # best_model = model.estimate()
-        best_model = model.estimate(max_indegree=max_indegree, epsilon=epsilon, max_iter=max_iter, black_list=black_list, white_list=white_list)
+        best_model = model.estimate(max_indegree=max_indegree, epsilon=epsilon, max_iter=max_iter, black_list=black_list, white_list=white_list, show_progress=False)
     else:
         # At this point, variables are readily filtered based on bw_list_method or not (if nothing defined).
-        best_model = model.estimate(max_indegree=max_indegree, epsilon=epsilon, max_iter=max_iter)
+        best_model = model.estimate(max_indegree=max_indegree, epsilon=epsilon, max_iter=max_iter, show_progress=False)
 
     # Store
     out['model']=best_model
@@ -346,7 +383,7 @@ def _exhaustivesearch(df, scoretype='bic', return_all_dags=False, verbose=3):
     out=dict()
 
     # Set scoring type
-    scoring_method = _SetScoringType(df, scoretype)
+    scoring_method = _SetScoringType(df, scoretype, verbose=verbose)
     # Exhaustive search across all dags
     model = ExhaustiveSearch(df, scoring_method=scoring_method)
     # Compute best DAG
